/* parse.c 

   Routines for parsing command lines from players and handling
   commands.

 Chris Eleveld      (The Insane Hermit)
 Robert Slaven      (The Celestial Knight/Tigger)
 Jerry Bolton, Jr.  (Creel)
 Dave Mott          (Energizer Rabbit)

*/
/* take the following out of comment to enable passwords. */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#ifdef POSIX
#  include <sys/select.h>
#endif
#ifdef STRICT_BSD
#  include <strings.h>
#else
#  include <string.h>
#endif
#include <ctype.h>
#include "lorien.h"
#include "newpass.c"

#define bad_comm_prompt ">> error:  Unrecognized command.  Type /? for help.\r\n"
#define IVCMD_SYN ">> error:  Invalid command syntax type /? for help.\r\n"
#define NO_PERM ">> error:  permission denied.\r\n"
#define DEAD_MSG ">> fatal:  You've been killed.\r\n"
#define BEEPS_MSG ">> /p beeps enabled.\r\n"
#define NOBEEPS_MSG ">> /p beeps disabled.\r\n"
#define NAME_MSG ">> Name changed.\r\n"
#define YELL_MSG ">> error:  You are in hush mode.  You cannot send or receive yells.\r\n"
#define MESSAGE_MSG ">> Arrival and departure messages enabled.\r\n"
#define NOMESSAGE_MSG ">> Arrival and departure message disabled.\r\n"
#define NO_CHAN_MSG ">> error:  Channel does not exist and cannot be created.\r\n"  
#define NO_CHAN_CHANGE_MSG ">> warning:  Channel unchanged.\r\n"
#define HUSH_MSG ">> You are now in hush mode.  Yells will be suppressed.\r\n"
#define UNHUSH_MSG ">> You are no longer in hush mode.\r\n"
#define ECHO_MSG ">> /p echoing enabled.\r\n"
#define NOECHO_MSG ">> /p echoing disabled.\r\n"
#define EXIT_MSG ">> Hope you enjoyed your stay.\r\n"
#define BEEPS_NEWLINE "\007\r\n"
#define NOBEEPS_NEWLINE "\r\n"
#define IS_SECURE_MSG ">> error:  That channel is secure.\r\n"
#define SECURE_MSG ">> Channel secured by (%d) %s.\r\n"
#define UNSECURE_MSG ">> Channel unsecured by (%d) %s.\r\n"
#define NO_SECURE_MSG ">> error:  You cannot secure the main channel.\r\n"
#define NO_LEVEL_MSG ">> Your security level will not show.\r\n"
#define LEVEL_MSG ">> Your security level will show.\r\n"
#define WRAP_MSG ">> VT200-style Auto-wrap enabled.\r\n"
#define NO_WRAP_MSG ">> Auto-wrap disabled.\r\n"
#define SCREAM_MSG ">> Yell mode enabled.\r\n"
#define NOSCREAM_MSG ">> Yell mode disabled.\r\n"
#define BAN_NOTFOUND ">> error:  pattern matches 0 ban list entries.\r\n"

extern int   die();
extern int   changelevel();
extern chan *findchannel();
extern chan *newchannel();
extern chan *channels;
extern char *expand_hilite();
extern char *ban_remove();

extern struct splayer *lookup();
extern struct splayer *players;
extern int             numconnect;
extern int             numconnected();

char sendbuf[OBUFSIZE];

struct splayer *pplayer;
char *command;

int timebuffer;

/* get timestamp now */
int
get_timestamp()
{
    time_t tmptime;
    
    tmptime = time((time_t *) 0);
    timebuffer = tmptime;

    return 1;
}

/* password code starts here. */
void 
enablePassword(pplayer, buf)    /* case 'E': */
    struct splayer *pplayer;
    char *buf;

{
    if (pplayer->seclevel < SUPREME) {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    }
    else if (!enablepass(buf)) {
	sprintf(sendbuf, IVCMD_SYN);
	sendtoplayer(pplayer, sendbuf);
    }
    else {
	sprintf(sendbuf,">> Password enabled for player.\r\n");
	sendtoplayer(pplayer, sendbuf);
	SET_VRFY(pplayer);
    };
}

void 
changePassword(pplayer, buf)  /* case 'P': */
    struct splayer *pplayer;
    char *buf;

{
    if (!VRFY(pplayer)) {
	sprintf(sendbuf,bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    }
    else {
	if (!newpass(pplayer->name, buf)) {
	    sprintf(sendbuf,IVCMD_SYN);
	    sendtoplayer(pplayer, sendbuf);
	}
	else {
	    sprintf(sendbuf,">> Password changed on Lorien.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	};
    }; 
}

void 
delete_player(pplayer, buf)  /* case 'D': */
    struct splayer *pplayer;
    char *buf;
{
    if (pplayer->seclevel >= SUPREME) {
	if (pfree(buf+1)) {
	    passwrite();
	    sprintf(sendbuf,">> Player deleted from database.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	}
	else {
	    sprintf(sendbuf,">> Name not found in database.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	};
    }
    else {
	sprintf(sendbuf,bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    }; 
}

void 
set_name_password(pplayer, buf, buf2, pstruct) /* case 'n': */ 
    struct splayer *pplayer;
    char *buf;
    char *buf2;
    struct p_word pstruct;
{
    buf++;

    if (!(pplayer->privs & CANNAME)) {
	sendtoplayer(pplayer,NO_PERM);
	return;
    }
    buf = (char *) skipspace(buf);
    buf2 = strchr(buf,'=');
		
    if (buf2 != (char *)0) {
	buf2[0] = 0x0;
	buf2++;
	strncpy(pstruct.name, buf, MAX_NAME-1);
	if (!plookup(&pstruct)) {
	    sprintf(sendbuf,">> Name not found in database.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	} else {
	    if (strcmp(pstruct.pword,buf2)) {
		sprintf(sendbuf,">> Invalid password.\r\n");
		sendtoplayer(pplayer, sendbuf);
	    } else {
		SET_VRFY(pplayer);
		setname(pplayer->s,buf);
		sprintf(sendbuf,">> Name changed.\r\n");
		sendtoplayer(pplayer, sendbuf);
	    };
	};
    } else {
	strncpy(pstruct.name, buf, MAX_NAME-1);
	if (!plookup(&pstruct)) {
	    setname(pplayer->s, buf);
	    CLR_VRFY(pplayer);
	    sprintf(sendbuf,">> Name changed.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	} else {
	    sprintf(sendbuf, ">> That name is reserved.  Please use another.\r\n");
	    sendtoplayer(pplayer, sendbuf);
	};
    };
}

/* password code ends here. */

void 
pose_it(pplayer, buf)
    struct splayer *pplayer;
    char *buf;
{
    buf++;
    if (*buf == ',' || *buf == '\'') 
	sprintf(sendbuf,"(%d) %s%s\r\n",
		pplayer->s,
		pplayer->name, buf);
    else
	sprintf(sendbuf,"(%d) %s %s\r\n", 
		pplayer->s,
		pplayer->name, buf);
    sendall(sendbuf, pplayer->chnl,0);
}

typedef enum { SPEECH_NORMAL, SPEECH_YELL, SPEECH_PRIVATE } speechmode_t;

void
stagepose(pplayer,buf, mode)
    struct splayer *pplayer;
    char *buf;
    speechmode_t mode;
{
    char buf2[2048];
    char hbuf[40];
    char hbuf2[40];
    static char fmt_posespace[40]="(%d) [to %s] %s %s\r\n";
    static char fmt_posenospace[40]="(%d) [to %s] %s%s\r\n";
    static char fmt_yellposespace[40]="(*%d*) [to %s] %s %s\r\n";
    static char fmt_yellposenospace[40]="(*%d*) [to %s] %s%s\r\n";
    static char fmt_yellstage[40]="(*%d*) %s [to %s] %s\r\n";
    static char fmt_stage[40]="(%d) %s [to %s] %s\r\n";
    char *fmtstring;
    int linenum;
    struct splayer *who, *next;

    if (pplayer->seclevel >= JOEUSER) {
	if (isdigit(buf[1])) {

	    linenum = atoi(buf + 1);
	    buf = (char *) skipdigits(buf + 1);

	    who = lookup(linenum);
	    if (who == (struct splayer *)0) {
		sprintf(sendbuf, ">> error:  Player %d does not exist.\r\n", linenum);
		sendtoplayer(pplayer, sendbuf);
	    } else {
		buf = (char *) skipspace(buf);
		if(*buf == ':' || *buf == ';') {
		    buf++;
		    buf = (char *) skipspace(buf);
		    if (*buf == ',' || *buf == '\'')
			switch(mode) {
                            case SPEECH_NORMAL:
			        fmtstring = fmt_posenospace;
                                break;
			    case SPEECH_YELL:
			    	fmtstring = fmt_yellposenospace;
				break;
                        }
		    else
			switch(mode) {
                            case SPEECH_NORMAL:
			        fmtstring = fmt_posespace;
                                break;
			    case SPEECH_YELL:
			    	fmtstring = fmt_yellposespace;
				break;
                        }
		    sprintf(sendbuf,fmtstring,
			    pplayer->s,
			    who->name,
			    pplayer->name,
			    buf);
		    if (pplayer->seclevel > JOEUSER)
			sendall(sendbuf,pplayer->chnl,0);
		    else
			sendtoplayer(pplayer, sendbuf);
		} else {
		    buf = (char *) skipspace(buf);
		    fmtstring = (mode==SPEECH_YELL) ? fmt_yellstage : fmt_stage;
		    sprintf(sendbuf,fmtstring,
			    pplayer->s,
			    pplayer->name,
			    who->name,
			    buf);
                    if (pplayer->seclevel > JOEUSER) {
			if (mode == SPEECH_YELL) {
			    who = players->next;
			    while (who) {
			        next = who->next;
				sendtoplayer(who, sendbuf);
				who = next;
			    }
			}
			else
			    sendall(sendbuf,pplayer->chnl,0);
		    }
		    else
		        sendtoplayer(pplayer, sendbuf);
		}
	    }
	} else {
	    sprintf(sendbuf, bad_comm_prompt);
	    sendtoplayer(pplayer, sendbuf);
	}
    }
}

void
force(pplayer, buf)  /* case 'F': */
    struct splayer *pplayer;
    char *buf;
{
    if (isdigit(buf[1]) && (pplayer->seclevel >= SUPREME)) {
	int linenum;
	struct splayer *who;

	linenum = atoi(buf + 1);
	buf = (char *) skipdigits(buf + 1);

	who = lookup(linenum);
	if (who == (struct splayer *)0) {
	    sprintf(sendbuf, ">> error:  Player %d does not exist. \r\n", linenum);
	    sendtoplayer(pplayer, sendbuf);
	} else { 
	    if (who->seclevel < pplayer->seclevel) {
		buf = (char *) skipspace(buf);

		strcpy(who->pbuf,buf);
		processinput(who);
	    } else {
		sprintf(sendbuf, ">> %s has at least as much authority as you.\r\n", who->name);
		sendtoplayer(pplayer, sendbuf);
		sprintf(sendbuf, ">> %s just tried to force you...\r\n",
			pplayer->name);
		sendtoplayer(who, sendbuf);
	    }
	}
    } else {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    };
}

void 
gag_Player(pplayer, buf, line_number)  /* case 'G': */
    struct splayer *pplayer;
    char *buf;
    int line_number;
{
    struct splayer *who;

    line_number = atoi(&buf[1]);
    who = lookup(line_number);

    if (!who) {
	sprintf(sendbuf,">> error:  Player %d does not exist!\r\n",line_number);
    }
    else if ((line_number >= 1) && (line_number < MAXCONN)) {
	if (FD_ISSET(line_number, &pplayer->gags)) {
	    FD_CLR(line_number, &pplayer->gags);
	    sprintf(sendbuf, ">> Player %d ungagged.\r\n", line_number);
	}
	else {
	    FD_SET(line_number, &pplayer->gags);
	    sprintf(sendbuf, ">> Player %d gagged.\r\n", line_number);
	}
    }
    else {
	sprintf(sendbuf, bad_comm_prompt);
    };

    sendtoplayer(pplayer, sendbuf);
}

void 
change_onfrom(pplayer,buf)
    struct splayer *pplayer;
    char *buf;
{
#ifndef ONFROM_ANY
    if (pplayer->seclevel >= SYSOP) {
#endif
	buf++;
	buf = (char *) skipspace(buf);
	if (!*buf)
	    sendtoplayer(pplayer,bad_comm_prompt);
	else {
	    strncpy(pplayer->onfrom,buf,MAX_NAME-1);
	    pplayer->onfrom[MAX_NAME-1] = (char) 0x0;
	    sprintf(sendbuf,">> Host information changed to %s\r\n",pplayer->onfrom);
	    sendtoplayer(pplayer,sendbuf);
	}
#ifndef ONFROM_ANY
    }
    else sendtoplayer(pplayer,bad_comm_prompt); 
#endif
}

void 
finger(pplayer, line)
    struct splayer *pplayer;
    int line;
{
    struct splayer *who;
 
    who = lookup(line);

    if (!who) {
	sprintf(sendbuf,">> error:  Player %d does not exist!\r\n",line);
	sendtoplayer(pplayer,sendbuf);
	return;
    }
   
    sprintf(sendbuf,">> Name: %s\r\n", who->name);
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> Channel: %s %s\r\n",who->chnl->name,
	    (who->chnl->secure) ? "(Secured)" : "");
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> On From: %s\r\n",who->host);
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> On From: %s\r\n",who->numhost);
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> Toggles enabled: %s %s %s %s %s \r\n",
	    HUSH(who) ? "hushed" : "" ,
	    BEEPS(who) ? "beeps" : "",
	    ECHO(who) ? "echos on" : "",
	    (who->hilite) ? "hilites" : "",
	    !MSG(who) ? "message suppress" : "");
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> End of info\r\n");
    sendtoplayer(pplayer,sendbuf);
    return;
}

void
change_channel(pplayer, buf)
    struct splayer *pplayer;
    char *buf;
{
    chan *oldc;
    chan *newc;
    struct splayer *who;
    int linenumber;

    buf++;
    if (!(pplayer->privs & CANCHANNEL)) {
	sendtoplayer(pplayer,NO_PERM);
	return;
    }
    if (isdigit(*buf) && (pplayer->seclevel >= SYSOP)) {
	    /* we want to change someone elses channel */

	linenumber = atoi(buf);

	buf = (char *) skipdigits(buf);

	if (!*buf) {
	    sendtoplayer(pplayer,bad_comm_prompt);
	    return;
	} 

	buf = (char *) skipspace(buf);
	if (!*buf) {
	    sendtoplayer(pplayer,bad_comm_prompt);
	    return;
	}

	who = lookup(linenumber);

	if (!who) {
	    sprintf(sendbuf,">> error:  Player %d does not exist!\r\n",linenumber);
	    sendtoplayer(pplayer,sendbuf);
	    return;
	}

	if (who->seclevel >= pplayer->seclevel) {
	    sprintf(sendbuf,">> %s has at least as much authority as you!\r\n",who->name);
	    sendtoplayer(pplayer,sendbuf);
	    sprintf(sendbuf,">> %s just tried to change your channel.\r\n",pplayer->name);
	    sendtoplayer(who,sendbuf);
	    return;
	}

	strcpy(who->pbuf,"/c");
	strcat(who->pbuf,buf);
	processinput(who);
	sprintf(sendbuf,">> (%d) %s placed on channel %s.\r\n",who->s, who->name,
		who->chnl->name);
	sendtoplayer(pplayer,sendbuf);

    }
#ifdef NO_NUMBER_CHANNELS
    else if (isdigit(buf[1]))
	sendtoplayer(pplayer, bad_comm_prompt);
#endif
    else {

	buf = (char *) skipspace(buf);

	if (!strlen(buf)) {
	    newc = channels;

	    sprintf(sendbuf,">> %-13s %-10s %-6s\r\n","Channel","# Users","Secure");
	    sendtoplayer(pplayer,sendbuf);
	    sendtoplayer(pplayer,">> -------------------------------\r\n");

	    while (newc) {
		sprintf(sendbuf,">> %-13s %-10d %-6s\r\n",
			newc->name,
			newc->count,
			(newc->secure) ? "Yes" : "No");
		sendtoplayer(pplayer,sendbuf);
		newc = newc->next;
	    }
	    return;
	}

	if (pplayer->seclevel < BABYCO) {
	    sendtoplayer(pplayer,">> Channel changed.\r\n");
	    return;
	};

	newc = findchannel(buf);

	if (newc == pplayer->chnl) {
	    sendtoplayer(pplayer,NO_CHAN_CHANGE_MSG);
	    return;
	}

	if (!newc)
	    newc = newchannel(buf);

	if (!newc) {
	    sendtoplayer(pplayer,NO_CHAN_MSG);
	    return;
	}

	if (newc->secure) {
	    sendtoplayer(pplayer,IS_SECURE_MSG);
	    return;
	}

	oldc = pplayer->chnl;
	oldc->count--;
      
	if (oldc->count <= 0) {
	    remove_channel(oldc);
	    oldc = (chan *) 0;
	}

	newc->count++;

	sprintf(sendbuf,">> (%d) %s has joined.\r\n",pplayer->s, pplayer->name);
	sendall(sendbuf,newc,0);

	pplayer->chnl = newc;

	sprintf(sendbuf,">> Channel changed.\r\n");
	sendtoplayer(pplayer,sendbuf);

	sprintf(sendbuf,">> (%d) %s has wandered off.\r\n", pplayer->s, pplayer->name);
	if (oldc) sendall(sendbuf,oldc,0);
        
    }

    return;
}

void 
set_name(pplayer, buf, buf2, pstruct) /* case 'n': */ 
    struct splayer *pplayer;
    char *buf;
    char *buf2;
{
    buf++;

    if (!(pplayer->privs & CANNAME)) {
	sendtoplayer(pplayer,NO_PERM);
	return;
    }

    buf = (char *) skipspace(buf);
    buf2 = strchr(buf,'=');

    if (buf2 != (char *)0) {
	buf2[0]=0x0;
	strncpy(pplayer->name, buf, MAX_NAME-1);
    } else {
	strncpy(pplayer->name, buf, MAX_NAME-1);
    };
    sprintf(sendbuf, NAME_MSG);
    sendtoplayer(pplayer, sendbuf);
    return;
}

void 
whisper(pplayer, buf)      /*  case 'p': */
    struct splayer *pplayer;
    char *buf;

{
    char buf2[2048];
    char hbuf[40];
    char hbuf2[40];
    static char formatposespace[20]="%s(%d,p) %s %s%s%s";
    static char formatposenospace[20]="%s(%d,p) %s%s%s%s";
    char *fmtstring;

    if (! (pplayer->privs & CANWHISPER)) {
	sendtoplayer(pplayer,NO_PERM);
	return;   
    }
    if (pplayer->seclevel > JOEUSER) {
	if (isdigit(buf[1])) {
	    int linenum;
	    struct splayer *who;

	    linenum = atoi(buf + 1);
	    buf = (char *) skipdigits(buf + 1);

	    who = lookup(linenum);
	    if (who == (struct splayer *)0) {
		sprintf(sendbuf, ">> error:  Player %d does not exist.\r\n", linenum);
		sendtoplayer(pplayer, sendbuf);
	    } else {
		buf = (char *) skipspace(buf);
		if(*buf == ':' || *buf == ';') {
		    buf++;
		    buf = (char *) skipspace(buf);
		    if (*buf == ',' || *buf == '\'')
			fmtstring = formatposenospace;
		    else
			fmtstring = formatposespace;
		    sprintf(sendbuf,fmtstring,
			    (who->hilite) ? expand_hilite(who->hilite,hbuf) : "",
			    pplayer->s,
			    pplayer->name, buf,
			    (who->hilite) ? expand_hilite(0L, hbuf2) : "",
			    BEEPS(who) ? BEEPS_NEWLINE : NOBEEPS_NEWLINE);
		    sprintf(buf2, ">> Pose sent to %s : %s %s\r\n", who->name, pplayer->name, buf);
		} else {
		    buf = (char *) skipspace(buf);
		    sprintf(sendbuf,"%s(%d,p %s) %s%s%s",
			    (who->hilite) ? expand_hilite(who->hilite,hbuf) : "",
			    pplayer->s,
			    pplayer->name, buf,
			    (who->hilite) ? expand_hilite(0L,hbuf2) : "",
			    BEEPS(who) ? BEEPS_NEWLINE : NOBEEPS_NEWLINE);
		    sprintf(buf2, ">> /p sent to %s : %s\r\n", who->name, buf);
		}
		if (!isgagged(who, pplayer->s))
		    sendtoplayer(who, sendbuf);

		if (!ECHO(pplayer)) {
		    sendtoplayer(pplayer, ">> /p sent.\r\n");
		}
		else {
		    sendtoplayer(pplayer, buf2);
		}
	    }
	} else {
	    sprintf(sendbuf, bad_comm_prompt);
	    sendtoplayer(pplayer, sendbuf);
	}
    } else {
	sprintf(sendbuf,">> /p sent.\r\n");
	sendtoplayer(pplayer, sendbuf);
    };
}

void 
change_privs(pplayer,buf)
    struct splayer *pplayer;
    char *buf;
{
    int line;
    struct splayer *who;

    if (pplayer->seclevel < SUPREME) {
	sendtoplayer(pplayer,bad_comm_prompt);
	return;
    }

    buf = (char *) skipspace(buf + 1);

    if (!isdigit(*buf)) {
	sprintf(sendbuf,">> Bad player number in command.\r\n");
	sendtoplayer(pplayer,sendbuf);
	return;
    }

    line = atoi(buf);

    who = lookup(line);

    if (!who) {
	sprintf(sendbuf,">> error:  Player %d does not exist!\r\n",line);
	sendtoplayer(pplayer,sendbuf);
	return;
    }

    if (who->seclevel >= pplayer->seclevel) {
	sprintf(sendbuf,">> %s has at least as much authority as you.\r\n",who->name);
	sendtoplayer(pplayer,sendbuf);
	sprintf(sendbuf,">> (%d) %s just tried to change your privileges.\r\n",
		pplayer->s,pplayer->name);
	sendtoplayer(who,sendbuf);
	return;
    }

    buf = (char *) skipspace(skipdigits(buf));

    while (*buf) {
	*buf = toupper(*buf);
	switch(*buf) {
	case 'Y': if (who->privs & CANYELL) {
	    sprintf(sendbuf,">> Player %d is not allowed to '\''yell.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANYELL;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now yell.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANYELL;
	}; break;
	case 'W': if (who->privs & CANWHISPER) {
	    sprintf(sendbuf,">> Player %d is not allowed to whisper.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANWHISPER;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now whisper.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANWHISPER;
	}; break;
	case 'N': if (who->privs & CANNAME) {
	    sprintf(sendbuf,">> Player %d is not allowed to set own name.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANNAME;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now set own name.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANNAME; 
	}; break;
	case 'Q': if (who->privs & CANQUIT) {
	    sprintf(sendbuf,">> Player %d is not allowed to quit.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANQUIT;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now quit.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANQUIT;
	}; break;
	case 'T': if (who->privs & CANCHANNEL) {
	    sprintf(sendbuf,">> Player %d is not allowed to change channels\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANCHANNEL;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now change channels.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANCHANNEL;
	}; break;
	case 'C': if (who->privs & CANCAPS) {
	    sprintf(sendbuf,">> Player %d is not allowed to use capital letters.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs &= ~CANCAPS;
	}
	else {
	    sprintf(sendbuf,">> Player %d may now use capital letters.\r\n",line);
	    sendtoplayer(pplayer,sendbuf);
	    who->privs |= CANCAPS;
	}; break;
	}
	buf++;
    }
}


void 
yell(pplayer, buf)  /*  case 'a': */ 
    struct splayer *pplayer;
    char *buf;
{
    char *fmtstring;
    static char formatposespace[20] = "(*%d*) %s %s\r\n";
    static char formatposenospace[20] = "(*%d*) %s%s\r\n";
    struct splayer *who,*next;

    if (!(pplayer->privs & CANYELL)) {
	sendtoplayer(pplayer,NO_PERM);
	return;
    }

    if (HUSH(pplayer)) {
	sprintf(sendbuf, YELL_MSG);
	sendtoplayer(pplayer, sendbuf);
	return;
    };

    if (pplayer->seclevel >= JOEUSER) { 

	buf = (char *) skipspace(buf);
	if (*buf == '`') {
	    stagepose(pplayer, buf, SPEECH_YELL);
	    return;
	}

	if(*buf == ':' || *buf == ';') {
	    buf = (char *) skipspace(buf + 1);

	    if (*buf == ',' || *buf == '\'')
		fmtstring=formatposenospace;
	    else
		fmtstring=formatposespace;

	    sprintf(sendbuf,fmtstring,
		    pplayer->s,
		    pplayer->name, buf);
	} else {
	    sprintf(sendbuf,"(*%d, %s*) %s\r\n",
		    pplayer->s,
		    pplayer->name, buf);
	}
	if ((pplayer->seclevel) > JOEUSER) {
	    who = players->next;
	    while (who) {
	        next = who->next;
		if (!HUSH(who) && !isgagged(who,pplayer->s))
		    sendtoplayer(who, sendbuf);
		who = next;
	    }
	}
        else /* Joe should think it was sent, but it wasnt */
	    sendtoplayer(pplayer, sendbuf);
    };
}

void 
echo(pplayer)
    struct splayer *pplayer;

{
    XOR_ECHO(pplayer);
    if (ECHO(pplayer)) {
	sprintf(sendbuf, ECHO_MSG);
    }
    else sprintf(sendbuf, NOECHO_MSG);
    sendtoplayer(pplayer, sendbuf);
}

void 
hush(pplayer)
    struct splayer *pplayer;

{
	/* if unhushed and screaming, can't scream no more. */
    if (SCREAM(pplayer) && !HUSH(pplayer)) {
	sprintf(sendbuf, NOSCREAM_MSG);
	sendtoplayer(pplayer, sendbuf);
	XOR_SCREAM(pplayer);
    }

    XOR_HUSH(pplayer);
    if (HUSH(pplayer)) {
	sprintf(sendbuf, HUSH_MSG);
    }
    else sprintf(sendbuf, UNHUSH_MSG);
    sendtoplayer(pplayer, sendbuf);
}

void
scream(pplayer)
    struct splayer *pplayer;

{
    if (HUSH(pplayer)) {
	sprintf(sendbuf, YELL_MSG);
	sendtoplayer(pplayer, sendbuf);
    }
    else {
	XOR_SCREAM(pplayer);
	if (SCREAM(pplayer)) 
	    sprintf(sendbuf, SCREAM_MSG);
	else
	    sprintf(sendbuf, NOSCREAM_MSG);
	sendtoplayer(pplayer, sendbuf);
    }
}

void 
promote(pplayer, buf)  /* case '+': */
    struct splayer *pplayer;
    char *buf;
{
    if (pplayer->seclevel >= SYSOP) {
	int linenum;
	struct splayer *who;

	linenum = atoi(buf + 1);
	buf = (char *) skipdigits(buf + 1);

	who = lookup(linenum);
	if (who == (struct splayer *)0) {
	    sprintf(sendbuf, ">> error:  Player %d does not exist.\r\n", linenum);
	    sendtoplayer(pplayer, sendbuf);
	} else {
	    if ((who->seclevel + 1) >= pplayer->seclevel) {
		sprintf(sendbuf, ">> %s's security level is too high.\r\n", who->name);
		sendtoplayer(pplayer, sendbuf);
	    } else {
		who->seclevel++;
		sprintf(sendbuf, "%s(%d) promoted %s(%d)",
			pplayer->name,pplayer->seclevel,who->name,who->seclevel);
		log(sendbuf);
		sprintf(sendbuf, ">> %s promoted.\r\n", who->name);
		sendtoplayer(pplayer, sendbuf);
		sprintf(sendbuf, ">> You have been promoted.\r\n");
		sendtoplayer(who, sendbuf);
	    }
	}
    } else {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    } 
}

void 
demote(pplayer, buf) /* case '-': */
    struct splayer *pplayer;
    char *buf;
{
    if (pplayer->seclevel >= SYSOP) {
	int linenum;
	struct splayer *who;

	linenum = atoi(buf + 1);
	buf = (char *) skipdigits(buf + 1);

	who = lookup(linenum);
	if (who == (struct splayer *)0) {
	    sprintf(sendbuf, ">> error:  Player %d does not exist.\r\n", linenum);
	    sendtoplayer(pplayer, sendbuf);
	} else {
	    if (who->seclevel < pplayer->seclevel) {
		who->seclevel--;
		sprintf(sendbuf, ">> %s demoted.\r\n", who->name);
		sendtoplayer(pplayer, sendbuf);
	    } else {
		sprintf(sendbuf,">> You don't have authority over %s!\r\n",who->name);
		sendtoplayer(pplayer, sendbuf);
		sprintf(sendbuf, ">> %s just tried to demote you!\r\n",pplayer->name);
		sendtoplayer(who, sendbuf);
	    }
	}
    } else {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    }
}

void 
kill_player(pplayer, buf)  /* case 'k': */
    struct splayer *pplayer;
    char *buf;
{
    if (pplayer->seclevel >= SYSOP) {
	int linenum;
	struct splayer *who;

	linenum = atoi(buf + 1);
	buf = (char *) skipdigits(buf + 1);

	who = lookup(linenum);
	if (who == (struct splayer *)0) {
	    sprintf(sendbuf, ">> error:  Player %d does not exist.\r\n", linenum);
	    sendtoplayer(pplayer, sendbuf);
	} else {
	    if (who->seclevel < pplayer->seclevel) {
		sprintf(sendbuf, DEAD_MSG);
		sendtoplayer(who, sendbuf);
		sprintf(sendbuf, ">> %s(%d) killed.\r\n", who->name, who->s);
		sendtoplayer(pplayer, sendbuf);
		sprintf(sendbuf, "%s killed %s.", pplayer->name, who->name);
		log(sendbuf);
		removeplayer(who);
	    } else {
		sprintf(sendbuf,">> You don't have authority over %s!\r\n",who->name);
		sendtoplayer(pplayer, sendbuf);
		sprintf(sendbuf, ">> %s just tried to kill you!\r\n", pplayer->name);
		sendtoplayer(who, sendbuf);
		sprintf(sendbuf, "%s tried to kill %s.", pplayer->name, who->name);
		log(sendbuf);
	    }
	}
    } else {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    };
}

void 
kill_all_players(pplayer, buf)  /* case 'K': */
    struct splayer *pplayer;
    char *buf;
{
    struct splayer *curr;
    struct splayer *next;

    if (pplayer->seclevel < SUPREME) {
	sendtoplayer(pplayer,bad_comm_prompt);
	return;
    }

    next = players->next;

    while (next) {
	curr = next;
	next = curr->next;
	if (curr->seclevel < SUPREME)
	    removeplayer(curr); 
    }
}

void 
broadcast(pplayer, buf)
    struct splayer *pplayer;
    char *buf;
{
    if (pplayer->seclevel >= SYSOP) {
	sprintf(sendbuf, ">> Broadcast message from (%d) %s : %s\r\n", pplayer->s, pplayer->name, &buf[1]);
	sendall(sendbuf, ALL,0);
    }
    else {
       
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    };
}

void 
broadcast2(pplayer, buf)
    struct splayer *pplayer;
    char *buf;
{
    int             line;
    struct splayer *who;

    if (pplayer->seclevel >= SUPREME) {
	buf = (char *) skipspace(buf + 1);
	line = atoi(buf);
	if (!line) {  
	    sprintf(sendbuf,"%s\r\n",buf);
	    sendall(sendbuf, INFORMATIONAL ,0);
	}
	else {
	    buf = (char *) skipspace(skipdigits(buf));
	    sprintf(sendbuf,"%s\r\n",buf);
	    who = lookup(line);
	    if (who)
		sendtoplayer(who, sendbuf);
	    else {
		sprintf(sendbuf,">> error:  Player %d does not exist.\r\n",line);
		sendtoplayer(pplayer,buf);
	    }
	}
    }
    else {
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
    };
}

int 
handlecommand(pplayer, command)
    struct splayer *pplayer;
    char *command;
{
    char *buf;
    char *buf2;
    struct p_word pstruct;
    int line_number;

    if (*command != '`')
	buf = command+1;
    else
	buf = command;


    passread();
	/* passopen(); */	/* set up password file. */

    switch(*(buf)) {
    default:
	sprintf(sendbuf, bad_comm_prompt);
	sendtoplayer(pplayer, sendbuf);
	break;
#ifdef PASSWORDS_ENABLED
    case 'E':   enablePassword(pplayer, buf); break;
#endif
    case 'P':   if (!strcmp("urgelog",(buf + 1)))
	if (!purgelog(pplayer))
	    sendtoplayer(pplayer,NO_PERM);
#ifdef PASSWORDS_ENABLED
	else
	    changePassword(pplayer, buf); 
    break;
    case 'D':   delete_player(pplayer, buf);  break;
#endif
    case 'F':   force(pplayer, buf);        break;
    case 'g':
	sendtoplayer(pplayer,">> Gags are /G as they should be.  Use /?G to read the help file.\r\n");
	break;
    case 'G':   gag_Player(pplayer, buf, line_number);     break;
    case 't':
    case 'c':   change_channel(pplayer, buf); break;
#ifndef PASSWORDS_ENABLED
    case 'n':   set_name(pplayer, buf, buf2, pstruct);  break;
#else
    case 'n':   set_name_password(pplayer, buf, buf2, pstruct); break;
#endif
    case '?':   showhelp(pplayer, &buf[1]); break;
    case 'w':
	buf = (char *) skipspace(buf + 1);
	if (*(buf) == '\000')
	    wholist(pplayer, (char *) 0);
	else
	    wholist(pplayer, (buf));
	break;
    case 'W':
	buf = (char *) skipspace(buf + 1);
	if (*(buf) == '\000')
	    wholist2(pplayer, (char *) 0);
	else
	    wholist2(pplayer, (buf));
	break;
    case 'f': 
	buf = (char *) skipspace(buf + 1);
	if (*buf) {

	    if (!isdigit(*buf)) {
		sendtoplayer(pplayer,bad_comm_prompt);
		break;
	    }
	    else {
		finger(pplayer,atoi(buf));
		break;
	    }                     
	}
	else
	    wholist3(pplayer);
	break; 
    case 'p': if (!strncmp(buf + 1, "lev",3)) {
	if (pplayer->seclevel >= SUPREME) {
	    printlevels(pplayer);
	    break;
	}
	else {
	    sendtoplayer(pplayer,bad_comm_prompt);
	    break;
	} 
    } else
	whisper(pplayer, buf);
    break;
    case 'q':
	pplayer->seclevel = changelevel(buf+1, pplayer);
          
	if (pplayer->seclevel == -1) {
            if (pplayer->privs & CANQUIT) {
#ifndef NO_LOG_CONNECT
		sprintf(sendbuf, "%s was on from %s",pplayer->name,pplayer->host);
		log(sendbuf);
#endif
		sprintf(sendbuf, EXIT_MSG);
		sendtoplayer(pplayer, sendbuf);
		SET_LEAVING(pplayer); /* this causes handleinput() to remove */
            }
            sprintf(sendbuf, NO_PERM);
            sendtoplayer(pplayer,sendbuf);
	}
	break;
    case 's': if (!strncmp(buf + 1,"hutdown",7)) {
#ifdef USE_SHUTDOWN_COMMAND
	if (pplayer->seclevel >= SUPREME)
	    haven_shutdown(pplayer);
	else
#endif
	    sendtoplayer(pplayer,bad_comm_prompt);
	break;
    }
    else {
	if (pplayer->chnl == channels) {
	    sendtoplayer(pplayer,NO_SECURE_MSG);
	    break;
	}
	if (pplayer->chnl->secure) {
	    sprintf(sendbuf,UNSECURE_MSG, pplayer->s, pplayer->name);
	    sendall(sendbuf,pplayer->chnl,0);
	}
	else {
	    sprintf(sendbuf,SECURE_MSG, pplayer->s, pplayer->name);
	    sendall(sendbuf,pplayer->chnl,0);
	}
	pplayer->chnl->secure = !pplayer->chnl->secure;
    }; break;
    case 'a':
    case 'y':  yell(pplayer, buf + 1);  break;
    case 'Y': scream(pplayer); break;
    case '+' : promote(pplayer, buf);  break;
    case '-' : demote(pplayer, buf);  break;
    case 'k' : kill_player(pplayer, buf);  break;
    case 'K' : kill_all_players(pplayer, buf);  break;
    case 'b' : if (!strncmp((buf + 1), "an",2)) {
	buf +=3;
	if (!strncmp(buf,"del",3)) {
	    char *s;
	    if (pplayer->seclevel < COSYSOP) {
		sendtoplayer(pplayer,bad_comm_prompt);
		break;
	    }
	    buf = (char *) skipspace(buf + 4);
	    s = ban_remove(buf);
	    if (s) {
		sprintf(sendbuf,">> %s removed from banlist.\r\n",s);
		sendall(sendbuf,ALL,pplayer->s);
	    }
	    else 
		sendtoplayer(pplayer,BAN_NOTFOUND);
	}
	else if (!strncmp(buf,"add",3)) {
	    if (pplayer->seclevel < COSYSOP) {
		sendtoplayer(pplayer,bad_comm_prompt);
		break;
	    }
	    buf = (char *) skipspace(buf + 4);
	    ban_add(buf);
	    sprintf(sendbuf,">> %s added to banlist.\r\n",buf);
		/* this read sendtoplayer(sendbuf,ARRIVAL,pplayer->s);
		   for the 1.6.2a distribution. */
	    sendall(sendbuf,ALL,pplayer->s);
	}
	else {
	    ban_list(pplayer);
	}
    }
    else {
	XOR_BEEPS(pplayer);
	if (BEEPS(pplayer))
	    sprintf(sendbuf,BEEPS_MSG);
	else
	    sprintf(sendbuf,NOBEEPS_MSG);
	sendtoplayer(pplayer,sendbuf);
    };
    break;
    case '`':
        stagepose(pplayer,buf,SPEECH_NORMAL);
    break;
    case 'M' : 
	if (*(buf + 1) == 'o') {
	    if (pplayer->seclevel >= SUPREME) {
		buf = (char *) skipspace(buf + 2);
		line_number = atoi(buf);
		if (line_number) 
		    MAXCONN = settablesize(line_number);
		else 
		    MAXCONN = gettablesize();
		sprintf(sendbuf,">> fd limit now %d.\r\n",MAXCONN);
	    }
	    else
		sprintf(sendbuf, NO_PERM);

	    sendtoplayer(pplayer, sendbuf);
	    break; 
	}
	else {
	    if (pplayer->seclevel >= SYSOP) {
		buf = (char *) skipspace(buf + 1);
		if (!*buf) {
		    sendtoplayer(pplayer,">> Invalid command syntax.\r\n");
		    break;
		}
		strncpy(channels->name,buf,MAX_CHAN);
		channels->name[MAX_CHAN - 1] = 0;
		sprintf(sendbuf,">> main channel now \"%s\"\r\n",channels->name);
		sendall(sendbuf,ARRIVAL,pplayer->s);
	    }
	    else {
		sendtoplayer(pplayer,bad_comm_prompt);
	    }
	}
	break;
    case 'm' : {
	XOR_MSG(pplayer);
	if (MSG(pplayer))
	    sprintf(sendbuf,MESSAGE_MSG);
	else
	    sprintf(sendbuf,NOMESSAGE_MSG);
	sendtoplayer(pplayer,sendbuf);
    }; break;
    case 'h' : buf++;
	if (*buf != 'i') {
	    hush(pplayer);
	    break;
	}
    case 'H' : buf++;
	if (construct_mask(buf,&pplayer->hilite) < 0) {
	    sprintf(sendbuf,">> Error parsing hilite string.\r\n");
	    sendtoplayer(pplayer,sendbuf);
	}
	else {
	    char tmpstr[BUFSIZE];
	    memset(tmpstr,0,BUFSIZE);
	    sprintf(sendbuf,"%s\r\n",
		    mask2string(pplayer->hilite,tmpstr,hi_types,","));
	    sendtoplayer(pplayer,sendbuf);
	}; break;

    case 'r' : XOR_WRAP(pplayer);
	if (WRAP(pplayer))
	    sendtoplayer(pplayer,WRAP_MSG);
	else
	    sendtoplayer(pplayer,NO_WRAP_MSG);
	break;
    case 'B' : broadcast(pplayer, buf); break;
    case 'i' : broadcast2(pplayer, buf); break;
    case 'e' : echo(pplayer); break;
    case 'o' : change_onfrom(pplayer, buf); break;
    case 'C' : change_privs(pplayer,buf); break;
    case 'u' : sprintf(sendbuf,">> Lorien %s has been running for %s.\r\n",
		       VERSION,
		       timelet(timebuffer,40000));
    sendtoplayer(pplayer,sendbuf);
    sprintf(sendbuf,">> %d of a possible %d lines are in use.\r\n",
	    numconnected(),
	    MAXCONN - 4);
    sendtoplayer(pplayer,sendbuf);
    break;
    case 'I' : XOR_INFO(pplayer);
	if (INFO(pplayer))
	    sprintf(sendbuf,">> /i messages enabled.\r\n");
	else
	    sprintf(sendbuf,">> /i messages suppressed.\r\n");

	sendtoplayer(pplayer,sendbuf);
	break;
                   
    case 'l' : {
	if (pplayer->seclevel < SYSOP) {
	    sendtoplayer(pplayer,bad_comm_prompt);
	    break;
	}

	XOR_SHOW(pplayer);

	if (!SHOW(pplayer))
	    sprintf(sendbuf,NO_LEVEL_MSG);
	else
	    sprintf(sendbuf,LEVEL_MSG);

	sendtoplayer(pplayer,sendbuf);
    }; break;
    };
}


